--[[
Interface: 1.5.1.0 b1580

Copyright (C) GtX (Andy), 2017

Author: GtX | Andy
Date: 16.06.2017
Version: 1.0.0.0

Support and Guide: https://ls-modcompany.com

Thank you to:
Rival and Manuel Leithner  -  Inspiration from past scripts.


History:
V 1.0.0.0 		@ 	16.06.2017 	- 	Project Started.
V 1.0.1.0	 	@ 	01.07.2017 	- 	Added Start Key animation option.
V 1.1.0.0	 	@ 	19.04.2018 	- 	Optimise script, add realLight option to drl, add support for driveControl 'shuttle' and 'differential icons',
									Added mph / kph display options depending on the player’s game settings.
									Handbrake can now either rotate or translate.
V 1.1.1.0	 	@ 	31.05.2018 	- 	Fix conflict when using Gearbox Addon Versions greater than V3.0.0.1 to V3.0.5.9.
									Future mrGB release versions after V3.0.5.9 should also work with this script no problem.
V 1.1.2.0 		@ 	23.06.2018 	- 	Added new switches functions. TYPES: Lights, Hazard lights and Beacon lights.

******************************************************************************************************************
** Important **
***************

frei verwendbar - keine erlaubnis nötig    /    Modifikationen erst nach Rücksprache!

free for use - no permission needed.       /    Modifications and Conversions only with my permission!

*******************************************************************************************************************
]]

realCab = {}

function realCab.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations) and SpecializationUtil.hasSpecialization(Drivable, specializations)
end

function realCab:load(savegame)
	self.setDrlState = realCab.setDrlState
	self.handbrakeStates = realCab.handbrakeStates
	self.setTurnLightState  = Utils.appendedFunction(self.setTurnLightState, realCab.setTurnLightState)
	self.setLightsTypesMask  = Utils.appendedFunction(self.setLightsTypesMask, realCab.setLightsTypesMask)
	self.setBeaconLightsVisibility  = Utils.appendedFunction(self.setBeaconLightsVisibility, realCab.setBeaconLightsVisibility)
	--self.vehicleStateCheck = realCab.vehicleStateCheck

	self.realCab = {}
	self.dashActive = false
	self.dashUpdate = false
	self.realCabGears = false
	self.driveControlFound = false
	local hasWipers = false

	if hasXMLProperty(self.xmlFile, "vehicle.realCab.onStart") then
		self.realCab.dash = {}
		local i = 0
		while true do
			local onStart = string.format("vehicle.realCab.onStart.startGroup(%d)", i)
			if not hasXMLProperty(self.xmlFile, onStart) then
				break
			end

			local dash = {}
			dash.floatTime = 1
			dash.hasLogo = false
			dash.hasWarn = false
			local hudDisplay = Utils.indexToObject(self.components, getXMLString(self.xmlFile, onStart.."#extraScreenDisplay"))	--Displayed with no wait time.
			if hudDisplay ~= nil then
				setVisibility(hudDisplay, false)
				dash.hudDisplay = hudDisplay
			end
			local logoDisplay = Utils.indexToObject(self.components, getXMLString(self.xmlFile, onStart.."#logoDisplay")) --Displayed before main screen.
			local screenDisplay = Utils.indexToObject(self.components, getXMLString(self.xmlFile, onStart.."#mainScreenDisplay")) --Displayed after logo.
			if logoDisplay ~= nil and screenDisplay ~= nil then
				setVisibility(logoDisplay, false)
				setVisibility(screenDisplay, false)
				dash.logoDisplay = logoDisplay
				dash.screenDisplay = screenDisplay
				dash.logoHoldTime = Utils.getNoNil(getXMLFloat(self.xmlFile, onStart.. "#logoDisplayTime"),3000) --Load hang time of logo. Base default 3 Seconds.
				dash.hasLogo = true
				dash.logoActive = false
			end
			local warnLightCheck = Utils.indexToObject(self.components, getXMLString(self.xmlFile, onStart.."#warnLightCheck")) --Display warn lights for set time on vehicle start.
			if warnLightCheck ~= nil then
				setVisibility(warnLightCheck, false)
				dash.warnLightCheck = warnLightCheck
				dash.warnHoldTime = Utils.getNoNil(getXMLFloat(self.xmlFile, onStart.. "#warnLightDisplayTime"),2000) --Load hang time of warnLights. Base default 2 Seconds.
				dash.hasWarn = true
				dash.warnActive = false
			end

			table.insert(self.realCab.dash, dash)
			i = i + 1
		end
	end

	if hasXMLProperty(self.xmlFile, "vehicle.realCab.cabAnimations") then
		self.realCab.ca = {}
		local wipers = "vehicle.realCab.cabAnimations"
		self.realCab.ca.wipersActivated = false
		self.realCab.ca.wipersAnimation = getXMLString(self.xmlFile, wipers.. ".autoWipers#animationName") --Autowiper animation name.
		if self.realCab.ca.wipersAnimation ~= nil then
			hasWipers = true
		end
		self.realCab.ca.wipersSpeed = Utils.getNoNil(getXMLFloat(self.xmlFile, wipers.. ".autoWipers#moveSpeed"),3)	--Autowiper speed of animations. (Default value is preset at 3 as it is a nice speed)
	end

	if hasXMLProperty(self.xmlFile, "vehicle.realCab.dayRunLights") then
		self.realCab.drl = {}
		self.dayRunLightState = false

		local i = 0
		while true do
			local drlKey = string.format("vehicle.realCab.dayRunLights.light(%d)", i)
			if not hasXMLProperty(self.xmlFile, drlKey) then
				break
			end
			local drl = {}
			local deco = Utils.indexToObject(self.components, getXMLString(self.xmlFile, drlKey.."#decoration")) --This is a minimum requirement and is the index of the corona.
			if deco ~= nil then
				setVisibility(deco, false)
				drl.deco = deco
				local realHigh = Utils.indexToObject(self.components, getXMLString(self.xmlFile, drlKey.."#realLightHigh")) -- This is optional and is the index of a real light. Only shown on High Graphic settings.

				if realHigh ~= nil then
					setVisibility(realHigh, false)
					drl.realHigh = realHigh
					drl.hasReal = true
				else
					drl.hasReal = false
				end

				table.insert(self.realCab.drl, drl)
			else
				print("Error: (realCab.lua) "..drlKey.."  -  'decoration' light missing in '"..self.configFileName.."'")
			end
			i = i + 1
		end
	end

	if hasXMLProperty(self.xmlFile, "vehicle.realCab.onStartAnimations") then
		self.realCab.osa = {}
		local active = "vehicle.realCab.onStartAnimations";
		self.realCab.osa.animationsOnStart = getXMLString(self.xmlFile, active..".onStartAnimation#name") --Animation name for General onStart Animations.
		self.realCab.osa.tempAnimations = getXMLString(self.xmlFile, active..".tempAnimation#name") --Animation name for fake Temp gauges
		self.realCab.osa.coolTime = Utils.getNoNil(getXMLFloat(self.xmlFile, active..".tempAnimation#tempCoolSpeed"),1) --Return modifier for tempAnimationName.
		self.realCab.osa.aosTime = Utils.getNoNil(getXMLFloat(self.xmlFile, active..".onStartAnimation#returnSpeed"),1) --Return modifier for onStartAnimationName.
	end

	if hasXMLProperty(self.xmlFile, "vehicle.realCab.realWarnLights") then
		self.realCab.iw = {}
		local warn = "vehicle.realCab.realWarnLights"

		local cruiseLight = Utils.indexToObject(self.components, getXMLString(self.xmlFile, warn..".cruiseControl#index")) --Display a Cruise Control active light.
		if cruiseLight ~= nil then
			setVisibility(cruiseLight, false)
			self.realCab.iw.cruiseLight = cruiseLight
			self.realCab.iw.cruiseLightOn = false
		end
		local noDetachLight = Utils.indexToObject(self.components, getXMLString(self.xmlFile, warn..".hitchNoDetach#index")) --Display a flashing warning light if trailer can not be detached.
		if noDetachLight ~= nil then
			setVisibility(noDetachLight, false)
			self.realCab.iw.noDetachLight = noDetachLight
			self.realCab.iw.noDetachLightOn = false
		end
		local canConnectLight = Utils.indexToObject(self.components, getXMLString(self.xmlFile, warn..".hitchCanConnect#index")) --Display a ready to attach light when near an implement or trailer.
		if canConnectLight ~= nil then
			setVisibility(canConnectLight, false)
			self.realCab.iw.canConnectLight = canConnectLight
			self.realCab.iw.canConnectLightOn = false
		end
		local beaconLights = Utils.indexToObject(self.components, getXMLString(self.xmlFile, warn..".beaconLights#index")) --Display a static Beacon active light.
		if beaconLights ~= nil then
			setVisibility(beaconLights, false)
			self.realCab.iw.beaconLights = beaconLights
			self.realCab.iw.beaconIconFlash = Utils.getNoNil(getXMLBool(self.xmlFile, warn..".beaconLights#flashDashIcon"), false) --Set if the Beacon Active light is static or flashing.
			self.realCab.iw.beaconLightsOn = false
		end
		if hasWipers then
			local wipersOnLight = Utils.indexToObject(self.components, getXMLString(self.xmlFile, warn..".wipersOn#index"))	--Displays a static Wipers active light if animated wipers are being used.
			if wipersOnLight ~= nil then
				setVisibility(wipersOnLight, false)
				self.realCab.iw.wipersOnLight = wipersOnLight
				self.realCab.iw.wipersOnLightOn = false
			end
		end
		local lowFuelLight = Utils.indexToObject(self.components, getXMLString(self.xmlFile, warn..".lowFuel#index")) --Displays a static Low Fuel light when fuel reaches set level.
		if lowFuelLight ~= nil then
			setVisibility(lowFuelLight, false)
			self.realCab.iw.lowFuelLight = lowFuelLight
			self.realCab.iw.fuelLowLevel = Utils.getNoNil(getXMLFloat(self.xmlFile, warn.. ".lowFuel#lowFuelLevel"),80)	--Sets at what level the Low Fuel Light will turn on.
			self.realCab.iw.lowFuelLightOn = false
		end

		--Drive Control and Gearbox Addon 4x4 and Diff Warning Lights
		local fourWheelIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, warn..".differentials#fourWheelDriveIcon")) -- Show an Icon when 4x4 mode is active
		if fourWheelIcon ~= nil then
			setVisibility(fourWheelIcon, false)
			self.realCab.iw.fourWheelIcon = fourWheelIcon
			self.realCab.iw.fourWheelIconOn = false
		end
		local diffLockFrontIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, warn..".differentials#diffLockFrontIcon")) -- Show an Icon when frontDiff is locked.
		if diffLockFrontIcon ~= nil then
			setVisibility(diffLockFrontIcon, false)
			self.realCab.iw.diffLockFrontIcon = diffLockFrontIcon
			self.realCab.iw.diffLockFrontIconOn = false
		end
		local diffLockBackIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, warn..".differentials#diffLockBackIcon")) -- Show an Icon when backDiff is locked.
		if diffLockBackIcon ~= nil then
			setVisibility(diffLockBackIcon, false)
			self.realCab.iw.diffLockBackIcon = diffLockBackIcon
			self.realCab.iw.diffLockBackIconOn = false
		end
	end

	if hasXMLProperty(self.xmlFile, "vehicle.realCab.insideActions") then
		local action = "vehicle.realCab.insideActions"
		self.realCab.ia = {}

		self.realCab.ia.turnIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, action..".indicators#index")) --Index of indicator stick.
		if self.realCab.ia.turnIndex ~= nil then
			self.realCab.ia.turnState = Lights.TURNLIGHT_OFF
			self.realCab.ia.leftTurn = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".indicators#leftTurn"), 3), {0, 0, 0}) --Position of indicator stick when turning left.
			self.realCab.ia.rightTurn = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".indicators#rightTurn"), 3), {0, 0, 0}) --Position of indicator stick when turning right.
			self.realCab.ia.turnOff = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".indicators#offPosition"), 3), {0, 0, 0}) --Position of indicator stick when off.
			setRotation(self.realCab.ia.turnIndex, self.realCab.ia.turnOff[1], self.realCab.ia.turnOff[2], self.realCab.ia.turnOff[3] )
		end

		if hasWipers then
			self.realCab.ia.wiperIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, action..".wipers#index")) --Index of wiper switch. (This function can only be used with the AutoWiper function)
			if self.realCab.ia.wiperIndex ~= nil then
				self.realCab.ia.wiperActive = false
				self.realCab.ia.wipersOff = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".wipers#offPosition"), 3), {0, 0, 0})
				self.realCab.ia.wipersOn = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".wipers#wipersOn"), 3), {0, 0, 0})
				setRotation(self.realCab.ia.wiperIndex, self.realCab.ia.wipersOff[1], self.realCab.ia.wipersOff[2], self.realCab.ia.wipersOff[3] )
			end
		end

		self.realCab.ia.handbrakeIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, action..".handbrake#index")) --Index of handbrake switch. Supports both standard and Gearbox Addon.
		if self.realCab.ia.handbrakeIndex ~= nil then
			self.realCab.ia.handbrakeActive = false
			self.realCab.ia.handbrakeRot = Utils.getNoNil(getXMLBool(self.xmlFile, action..".handbrake#useRotation"), true) -- Handbrake can be 'rotation' or 'translation' (x, y, z)
			if self.realCab.ia.handbrakeRot then
				self.realCab.ia.handbrakeOff = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".handbrake#offPosition"), 3), {0, 0, 0}) --Position of handbrake switch when off. (Rotate Mode)
				self.realCab.ia.handbrakeOn = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".handbrake#activated"), 3), {0, 0, 0}) --Position of handbrake switch when on. (Rotate Mode)
				setRotation(self.realCab.ia.handbrakeIndex, self.realCab.ia.handbrakeOff[1], self.realCab.ia.handbrakeOff[2], self.realCab.ia.handbrakeOff[3] )
			else
				self.realCab.ia.handbrakeOff = Utils.getNoNil(Utils.getVectorNFromString(getXMLString(self.xmlFile, action..".handbrake#offPosition"), 3), {0, 0, 0}) --Position of handbrake switch when off. (Translation Mode)
				self.realCab.ia.handbrakeOn = Utils.getNoNil(Utils.getVectorNFromString(getXMLString(self.xmlFile, action..".handbrake#activated"), 3), {0, 0, 0}) --Position of handbrake switch when on. (Translation Mode)
				setTranslation(self.realCab.ia.handbrakeIndex, self.realCab.ia.handbrakeOff[1], self.realCab.ia.handbrakeOff[2], self.realCab.ia.handbrakeOff[3] )
			end
		end

		self.realCab.ia.keyIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, action..".startKey#index"))	--Index of indicator stick.
		if self.realCab.ia.keyIndex ~= nil then
			self.realCab.ia.keyActive = false
			self.realCab.ia.engineOn = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".startKey#engineOn"), 3), {0, 0, 0}) --Position of key when engine is running.
			self.realCab.ia.engineOff = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".startKey#engineOff"), 3), {0, 0, 0}) --Position of key when engine is turned off.
			setRotation(self.realCab.ia.keyIndex, self.realCab.ia.engineOff[1], self.realCab.ia.engineOff[2], self.realCab.ia.engineOff[3] )
		end

		if hasXMLProperty(self.xmlFile, action..".switches") then
			local i = 0
			while true do
				local switchKey = string.format(action..".switches.lights.switch(%d)", i)
				if not hasXMLProperty(self.xmlFile, switchKey) then
					break
				end

				local switch = {}
				local lightType = getXMLInt(self.xmlFile, switchKey.."#lightType")
				if lightType ~= nil then
					if self.realCab.ia.lightSwitches == nil then
						self.realCab.ia.lightSwitches = {}
					end
					switch.type = lightType
					switch.index = Utils.indexToObject(self.components, getXMLString(self.xmlFile, switchKey.."#index"))
					switch.useRot = Utils.getNoNil(getXMLBool(self.xmlFile, switchKey.."#useRotation"), true)
					if switch.useRot then
						switch.on = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, switchKey.."#lightOn"), 3), {0, 0, 0})
						switch.off = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, switchKey.."#lightOff"), 3), {0, 0, 0})
						setRotation(switch.index, switch.off[1], switch.off[2], switch.off[3] )
					else
						switch.on = Utils.getNoNil(Utils.getVectorNFromString(getXMLString(self.xmlFile, switchKey.."#lightOn"), 3), {0, 0, 0})
						switch.off = Utils.getNoNil(Utils.getVectorNFromString(getXMLString(self.xmlFile, switchKey.."#lightOff"), 3), {0, 0, 0})
						setTranslation(switch.index, switch.off[1], switch.off[2], switch.off[3] )
					end
					switch.isActive = false
					table.insert(self.realCab.ia.lightSwitches, switch)
				else
					print("Error: (realCab.lua) "..switchKey.."  -  'lightType' not given in '"..self.configFileName.."'")
				end
				i = i + 1
			end

			self.realCab.ia.beaconIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, action..".switches.beaconLights#index"))
			if self.realCab.ia.beaconIndex ~= nil then
				self.realCab.ia.beaconSwitchRot = Utils.getNoNil(getXMLBool(self.xmlFile, action..".switches.beaconLights#useRotation"), true)
				if self.realCab.ia.beaconSwitchRot then
					self.realCab.ia.beaconOn = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".switches.beaconLights#beaconOn"), 3), {0, 0, 0})
					self.realCab.ia.beaconOff = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".switches.beaconLights#beaconOff"), 3), {0, 0, 0})
					setRotation(self.realCab.ia.beaconIndex, self.realCab.ia.beaconOff[1], self.realCab.ia.beaconOff[2], self.realCab.ia.beaconOff[3] )
				else
					self.realCab.ia.beaconOn = Utils.getNoNil(Utils.getVectorNFromString(getXMLString(self.xmlFile, action..".switches.beaconLights#beaconOn"), 3), {0, 0, 0})
					self.realCab.ia.beaconOff = Utils.getNoNil(Utils.getVectorNFromString(getXMLString(self.xmlFile, action..".switches.beaconLights#beaconOff"), 3), {0, 0, 0})
					setTranslation(self.realCab.ia.beaconIndex, self.realCab.ia.beaconOff[1], self.realCab.ia.beaconOff[2], self.realCab.ia.beaconOff[3] )
				end
			end

			self.realCab.ia.hazardIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, action..".switches.hazardLights#index"))
			if self.realCab.ia.hazardIndex ~= nil then
				self.realCab.ia.hazardSwitchRot = Utils.getNoNil(getXMLBool(self.xmlFile, action..".switches.hazardLights#useRotation"), true)
				self.realCab.ia.beaconState = Lights.TURNLIGHT_OFF
				if self.realCab.ia.hazardSwitchRot then
					self.realCab.ia.hazardOn = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".switches.hazardLights#hazardOn"), 3), {0, 0, 0})
					self.realCab.ia.hazardOff = Utils.getNoNil(Utils.getRadiansFromString(getXMLString(self.xmlFile, action..".switches.hazardLights#hazardOff"), 3), {0, 0, 0})
					setRotation(self.realCab.ia.hazardIndex, self.realCab.ia.hazardOff[1], self.realCab.ia.hazardOff[2], self.realCab.ia.hazardOff[3] )
				else
					self.realCab.ia.hazardOn = Utils.getNoNil(Utils.getVectorNFromString(getXMLString(self.xmlFile, action..".switches.hazardLights#hazardOn"), 3), {0, 0, 0})
					self.realCab.ia.hazardOff = Utils.getNoNil(Utils.getVectorNFromString(getXMLString(self.xmlFile, action..".switches.hazardLights#hazardOff"), 3), {0, 0, 0})
					setTranslation(self.realCab.ia.hazardIndex, self.realCab.ia.hazardOff[1], self.realCab.ia.hazardOff[2], self.realCab.ia.hazardOff[3] )
				end
			end
		end
	end

	if hasXMLProperty(self.xmlFile, "vehicle.realCab.hudDisplay") then
		local hudDisplay = "vehicle.realCab.hudDisplay"
		self.realCab.hd = {}
		self.realCab.hd.bcAnimations = VehicleHudUtils.loadHud(self, self.xmlFile, "brakeCompressor", hudDisplay)	--Adds a Analogue or Digital display for brakeCompressor fill volume using the standard indoorHud format.

		local mphIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, hudDisplay..".speedDisplayType#mphIcon")) -- Show a mph icon if this is the users setting.
		local kphIcon = Utils.indexToObject(self.components, getXMLString(self.xmlFile, hudDisplay..".speedDisplayType#kphIcon")) -- Show a kph icon if this is the users setting.
		if mphIcon ~= nil and kphIcon ~= nil then
			setVisibility(mphIcon, false)
			setVisibility(kphIcon, false)
			self.realCab.hd.mphIcon = mphIcon
			self.realCab.hd.kphIcon = kphIcon
			self.realCab.hd.useMiles = nil
			self.realCab.hd.loaded = true
		end
	end

	if hasXMLProperty(self.xmlFile, "vehicle.realCab.gearDisplay") then
		local autoGear = "vehicle.realCab.gearDisplay"
		self.realCab.gd = {}
		local groupIndex = Utils.indexToObject (self.components, getXMLString(self.xmlFile, "vehicle.realCab.gearDisplay#groupIndex")) --Main Group Index. Hides all displays if Gearbox Addon is active.
		if groupIndex ~= nil then
			setVisibility(groupIndex, false)
			self.realCab.gd.groupIndex = groupIndex
			self.realCab.gd.groupIndexOn = false
		end
		local backward = Utils.indexToObject(self.components, getXMLString(self.xmlFile, autoGear..".reverse#index")) --Display Reverse Icon when moving back.
		if backward ~= nil then
			setVisibility(backward, false)
			self.realCab.gd.reverse = backward
			self.realCab.gd.reverseOn = false
		end
		local drive = Utils.indexToObject(self.components, getXMLString(self.xmlFile, autoGear..".drive#index")) --Display Drive Icon when moving forward.
		if drive ~= nil then
			setVisibility(drive, false)
			self.realCab.gd.drive = drive
			self.realCab.gd.driveOn = false
		end
		local neutral = Utils.indexToObject(self.components, getXMLString(self.xmlFile, autoGear..".neutral#index")) --Display Neutral Icon when stopped.
		if neutral ~= nil then
			setVisibility(neutral, false)
			self.realCab.gd.neutral = neutral
			self.realCab.gd.neutralOn = false
		end
		if self.realCab.gd.groupIndex ~= nil then
			if not self.realCabGears then
				self.realCabGears = true
			end
		else
			if self.realCab.gd.reverse ~= nil then
				print("[Error::realCab.lua] reverse#index found but groupIndex is missing from vehicle.xml. Check vehicle.realCab.gearDisplay#groupIndex")
			end
			if self.realCab.gd.drive ~= nil then
				print("[Error::realCab.lua] drive#index found but groupIndex is missing from vehicle.xml. Check vehicle.realCab.gearDisplay#groupIndex")
			end
			if self.realCab.gd.neutral ~= nil then
				print("[Error::realCab.lua] neutral#index found but groupIndex is missing from vehicle.xml. Check vehicle.realCab.gearDisplay#groupIndex")
			end
		end
	end

	if hasXMLProperty(self.xmlFile, "vehicle.realCab.mrGbHud") then
		local gearboxGroup = "vehicle.realCab.mrGbHud"
		self.realCab.mrGb = {}
		local groupIndex = Utils.indexToObject (self.components, getXMLString(self.xmlFile, "vehicle.realCab.mrGbHud#groupIndex")) --Main Group Index. Hides all MRGB displays if Gearbox Addon is disabled.
		if groupIndex ~= nil then
			setVisibility(groupIndex, false)
			self.realCab.mrGb.groupIndex = groupIndex
			self.realCab.mrGb.groupIndexOn = false
		end
		self.realCab.mrGb.gearNumber = VehicleHudUtils.loadHud(self, self.xmlFile, "gearNumbers", gearboxGroup)	--Adds a Digital ONLY display for MrGB Mod Gear Number using the standard indoorHud format.
		local reverseIconIndex = Utils.indexToObject (self.components, getXMLString(self.xmlFile, "vehicle.realCab.mrGbHud.iconIndex#reverse"))	--"Reverse" Icon Index (Displayed when Reverse is activated in MRGBM).
		if reverseIconIndex ~= nil then
			setVisibility(reverseIconIndex, false)
			self.realCab.mrGb.reverseIconIndex = reverseIconIndex
			self.realCab.mrGb.reverseIconOn = false
		end
		local holdIconIndex = Utils.indexToObject (self.components, getXMLString(self.xmlFile, "vehicle.realCab.mrGbHud.iconIndex#autoHold")) --"Neutral" Icon Index (Displayed when Neutral is activated in MRGBM).
		if holdIconIndex ~= nil then
			setVisibility(holdIconIndex, false)
			self.realCab.mrGb.holdIconIndex = holdIconIndex
			self.realCab.mrGb.holdIconOn = false
		end
		if self.realCab.mrGb.groupIndex ~= nil then
			if not self.realCabGears then
				self.realCabGears = true
			end
		else
			if self.realCab.mrGb.gearNumber ~= nil then
				print("[Error::realCab.lua] gearNumbers found but groupIndex missing from vehicle.xml. Check vehicle.realCab.mrGbHud#groupIndex")
			end
			if self.realCab.mrGb.reverseIconIndex ~= nil then
				print("[Error::realCab.lua] iconIndex#reverse found but groupIndex missing from vehicle.xml. Check vehicle.realCab.mrGbHud#groupIndex")
			end
			if self.realCab.mrGb.holdIconIndex ~= nil then
				print("[Error::realCab.lua] iconIndex#autoHold found but groupIndex missing from vehicle.xml. Check vehicle.realCab.mrGbHud#groupIndex")
			end
		end

		if self.realCab.mrGb.gearNumber ~= nil then
			if self.realCab.mrGb.reverseIconIndex == nil then
				print("[Error::realCab.lua] gearNumbers found but iconIndex#reverse is missing from the vehicle.xml. Check vehicle.realCab.mrGbHud.iconIndex#reverse")
			end
			if self.realCab.mrGb.holdIconIndex == nil then
				print("[Error::realCab.lua] gearNumbers found but iconIndex#autoHold is missing from the vehicle.xml. Check vehicle.realCab.mrGbHud.iconIndex#autoHold")
			end
		end
	end
end

function realCab:delete()
end

function realCab:readStream(streamId, connection)
end

function realCab:writeStream(streamId, connection)
end

function realCab:update(dt)
	if self.isClient then

		local gearboxIsOn = false
		if self.mrGbMS ~= nil and self.mrGbMS.IsOn ~= nil then
			gearboxIsOn = self.mrGbMS.IsOn
		end

		local wipersActive = false
		if self.realCab.ca ~= nil then
			wipersActive = self.isMotorStarted and self.realCab.ca.wipersActivated
		end

		local dcShuttleOn = false
		if self.driveControl ~= nil and self.driveControl.shuttle ~= nil then
			if self.driveControl.shuttle.isActive then
				dcShuttleOn = true
			end
		end

		if self.realCab.dash ~= nil or self.realCab.osa ~= nil then
			if self.isMotorStarted ~= self.dashActive or self.dashUpdate then

				if self.realCab.dash ~= nil then
					local doDashCheck = false
					for i=1, table.getn(self.realCab.dash) do
						local dash = self.realCab.dash[i]

						if dash.hudDisplay ~= nil then
							setVisibility(dash.hudDisplay, self.isMotorStarted)
						end

						if dash.logoDisplay ~= nil and dash.screenDisplay ~= nil then
							if self.isMotorStarted then
								if dash.floatTime < dash.logoHoldTime then
									setVisibility(dash.logoDisplay, true)
									setVisibility(dash.screenDisplay, false)
									dash.logoActive = true
								end
								if dash.floatTime < (dash.logoHoldTime + dash.logoHoldTime) and dash.floatTime > dash.logoHoldTime then
									setVisibility(dash.logoDisplay, false)
									setVisibility(dash.screenDisplay, true)
									dash.logoActive = false
								end
								dash.floatTime = dash.floatTime + dt
								doDashCheck = true
							else
								dash.floatTime = 1
								setVisibility(dash.logoDisplay, false)
								setVisibility(dash.screenDisplay, false)
							end
						end

						if dash.warnLightCheck ~= nil then
							if self.isMotorStarted then
								if dash.floatTime < dash.warnHoldTime then
									setVisibility(dash.warnLightCheck, true)
									dash.warnActive = true
								end
								if dash.floatTime < (dash.warnHoldTime + dash.warnHoldTime) and dash.floatTime > dash.warnHoldTime then
									setVisibility(dash.warnLightCheck, false)
									dash.warnActive = false
								end
								dash.floatTime = dash.floatTime + dt
								doDashCheck = true
							else
								dash.floatTime = 1
								setVisibility(dash.warnLightCheck, false)
							end
						end
					end

					if doDashCheck then
						self.dashUpdate = false
						for _, child in pairs (self.realCab.dash) do
							if (child.hasLogo and child.logoActive) or (child.hasWarn and child.warnActive) then
								self.dashUpdate = true
								break
							end
						end
					end
				end

				if self.realCab.osa ~= nil then
					if self.realCab.osa.animationsOnStart ~= nil then
						local aos = self.realCab.osa
						if self.isMotorStarted then
							self:playAnimation(aos.animationsOnStart, 1, self:getAnimationTime(aos.animationsOnStart), true)
						else
							self:playAnimation(aos.animationsOnStart, -aos.aosTime, self:getAnimationTime(aos.animationsOnStart), true)
						end
					end

					if self.realCab.osa.tempAnimations ~= nil then
						local ta = self.realCab.osa
						if self.isMotorStarted then
							self:playAnimation(ta.tempAnimations, 1, self:getAnimationTime(ta.tempAnimations), true);
						else
							self:playAnimation(ta.tempAnimations, -ta.coolTime, self:getAnimationTime(ta.tempAnimations), true)
						end
					end
				end

				self.dashActive = self.isMotorStarted
			end
		end

		if self.realCab.ia ~= nil then
			local ia = self.realCab.ia
			if ia.wiperIndex ~= nil then
				if wipersActive then
					if not ia.wiperActive then
						setRotation(ia.wiperIndex, ia.wipersOn[1], ia.wipersOn[2], ia.wipersOn[3] )
						ia.wiperActive = true
					end
				else
					if ia.wiperActive then
						setRotation(ia.wiperIndex, ia.wipersOff[1], ia.wipersOff[2], ia.wipersOff[3] )
						ia.wiperActive = false
					end
				end
			end

			if ia.handbrakeIndex ~= nil then
				if self.isMotorStarted and gearboxIsOn then
					if self.mrGbMS.Handbrake ~= nil then
						if self.mrGbMS.Handbrake then
							if not ia.handbrakeActive then
								self:handbrakeStates(true)
							end
						else
							if ia.handbrakeActive then
								self:handbrakeStates(false)
							end
						end
					end
				elseif self.isMotorStarted and not gearboxIsOn then
					if ia.handbrakeIndex ~= nil then
						if self.movingDirection == 0 then
							if not ia.handbrakeActive then
								self:handbrakeStates(true)
							end
						else
							if ia.handbrakeActive then
								self:handbrakeStates(false)
							end
						end
					end
				elseif not self.isMotorStarted and self.movingDirection == 0 then
					if ia.handbrakeIndex ~= nil then
						if not ia.handbrakeActive then
							self:handbrakeStates(true)
						end
					end
				end
			end

			if ia.keyIndex ~= nil then
				if self.isMotorStarted then
					if not ia.keyActive then
						setRotation(ia.keyIndex, ia.engineOn[1], ia.engineOn[2], ia.engineOn[3] )
						ia.keyActive = true
					end
				else
					if ia.keyActive then
						setRotation(ia.keyIndex, ia.engineOff[1], ia.engineOff[2], ia.engineOff[3] )
						ia.keyActive = false
					end
				end
			end
		end

		if self.realCab.iw ~= nil then
			local iw = self.realCab.iw
			if iw.cruiseLight ~= nil then
				if self.isMotorStarted then
					if self.cruiseControl.state ~= Drivable.CRUISECONTROL_STATE_OFF then
						if not iw.cruiseLightOn then
							setVisibility(iw.cruiseLight, true)
							iw.cruiseLightOn = true
						end
					else
						if iw.cruiseLightOn then
							setVisibility(iw.cruiseLight, false)
							iw.cruiseLightOn = false
						end
					end
				else
					if iw.cruiseLightOn then
						setVisibility(iw.cruiseLight, false)
						iw.cruiseLightOn = false
					end
				end
			end

			if iw.noDetachLight ~= nil and self.attacherJoints ~= nil then
				local warnFlash = Utils.clamp(math.cos( 9.0 * (g_currentMission.time/1000) ) + 0.2, 0, 1)
				if self.showDetachingNotAllowedTime > 0 and warnFlash == 0 then
					if not iw.noDetachLightOn then
						setVisibility(iw.noDetachLight, true)
						iw.noDetachLightOn = true
					end
				else
					if iw.noDetachLightOn then
						setVisibility(iw.noDetachLight, false)
						iw.noDetachLightOn = false
					end
				end
			end

			if iw.canConnectLight ~= nil and self.attacherJoints ~= nil then
				if g_currentMission.attachableInMountRange ~= nil then
					if not iw.canConnectLightOn then
						setVisibility(iw.canConnectLight, true)
						iw.canConnectLightOn = true
					end
				else
					if iw.canConnectLightOn then
						setVisibility(iw.canConnectLight, false)
						iw.canConnectLightOn = false
					end
				end
			end

			if iw.beaconLights ~= nil then
				if iw.beaconIconFlash then
					local warnFlash = Utils.clamp(math.cos( 9.0 * (g_currentMission.time/1000) ) + 0.2, 0, 1)
					if self.beaconLightsActive and warnFlash == 0 then
						if not iw.beaconLightsOn then
							setVisibility(iw.beaconLights, true)
							iw.beaconLightsOn = true
						end
					else
						if iw.beaconLightsOn then
							setVisibility(iw.beaconLights, false)
							iw.beaconLightsOn = false
						end
					end
				else
					if self.beaconLightsActive then
						if not iw.beaconLightsOn then
							setVisibility(iw.beaconLights, true)
							iw.beaconLightsOn = true
						end
					else
						if iw.beaconLightsOn then
							setVisibility(iw.beaconLights, false)
							iw.beaconLightsOn = false
						end
					end
				end
			end

			if iw.wipersOnLight ~= nil then
				if wipersActive then
					if not iw.wipersOnLightOn then
						setVisibility(iw.wipersOnLight, true)
						iw.wipersOnLightOn = true
					end
				else
					if iw.wipersOnLightOn then
						setVisibility(iw.wipersOnLight, false)
						iw.wipersOnLightOn = false
					end
				end
			end

			if iw.lowFuelLight ~= nil then
				if self.isMotorStarted then
					if self.fuelFillLevel <= iw.fuelLowLevel then
						if not iw.lowFuelLightOn then
							setVisibility(iw.lowFuelLight, true)
							iw.lowFuelLightOn = true
						end
					else
						if iw.lowFuelLightOn then
							setVisibility(iw.lowFuelLight, false)
							iw.lowFuelLightOn = false
						end
					end
				else
					if iw.lowFuelLightOn then
						setVisibility(iw.lowFuelLight, false)
						iw.lowFuelLightOn =  false
					end
				end
			end

			if gearboxIsOn or self.driveControl ~= nil then
				if iw.fourWheelIcon ~= nil or iw.diffLockFrontIcon ~= nil or iw.diffLockBackIcon ~= nil then
					local driveControlModule, fourWheelActive, diffLockFrontActive, diffLockBackActive = false, false, false, false
					if self.driveControl ~= nil then
						if self.dCcheckModule ~= nil and self:dCcheckModule("fourWDandDifferentials") then
							driveControlModule = true
							if self.driveControl.fourWDandDifferentials.fourWheel then
								fourWheelActive = true
							end
							if self.driveControl.fourWDandDifferentials.diffLockFront then
								diffLockFrontActive = true
							end
							if self.driveControl.fourWDandDifferentials.diffLockBack then
								diffLockBackActive = true
							end
						end
					end
					if not driveControlModule and gearboxIsOn then
						if self.mrGbMS.DiffLockMiddle ~= nil then
							if self.mrGbMS.DiffLockMiddle then
								fourWheelActive = true
							end
						end
						if self.mrGbMS.DiffLockFront ~= nil then
							if self.mrGbMS.DiffLockFront then
								diffLockFrontActive = true
							end
						end
						if self.mrGbMS.DiffLockBack ~= nil then
							if self.mrGbMS.DiffLockBack then
								diffLockBackActive = true
							end
						end
					end

					if iw.fourWheelIcon ~= nil then
						if fourWheelActive then
							if not iw.fourWheelIconOn then
								setVisibility(iw.fourWheelIcon, true)
								iw.fourWheelIconOn = true
							end
						else
							if iw.fourWheelIconOn then
								setVisibility(iw.fourWheelIcon, false)
								iw.fourWheelIconOn = false
							end
						end
					end

					if iw.diffLockFrontIcon ~= nil then
						if diffLockFrontActive then
							if not iw.diffLockFrontIconOn then
								setVisibility(iw.diffLockFrontIcon, true)
								iw.diffLockFrontIconOn = true
							end
						else
							if iw.diffLockFrontIconOn then
								setVisibility(iw.diffLockFrontIcon, false)
								iw.diffLockFrontIconOn = false
							end
						end
					end

					if iw.diffLockBackIcon ~= nil then
						if diffLockBackActive then
							if not iw.diffLockBackIconOn then
								setVisibility(iw.diffLockBackIcon, true)
								iw.diffLockBackIconOn = true
							end
						else
							if iw.diffLockBackIconOn then
								setVisibility(iw.diffLockBackIcon, false)
								iw.diffLockBackIconOn = false
							end
						end
					end
				end
			end
		end

		if self:getIsActive() then
			if self.isMotorStarted and self.realCabGears then
				if self.realCab.mrGb ~= nil and gearboxIsOn then
					if self.realCab.gd ~= nil then
						local gd = self.realCab.gd

						if gd.groupIndex ~= nil then
							if gd.groupIndexOn then
								setVisibility(gd.groupIndex, false)
								gd.groupIndexOn = false
							end
							if gd.neutral ~= nil and gd.neutralOn then
								setVisibility(gd.neutral, false)
								gd.neutralOn = false
							end
							if gd.reverse ~= nil and gd.reverseOn then
								setVisibility(gd.reverse, false)
								gd.reverseOn = false
							end
							if gd.drive ~= nil and gd.driveOn then
								setVisibility(gd.drive, false)
								gd.driveOn = false
							end
						end
					end

					local mrGb = self.realCab.mrGb
					if mrGb.groupIndex ~= nil then
						if not mrGb.groupIndexOn then
							setVisibility(mrGb.groupIndex, true)
							mrGb.groupIndexOn = true
						end
						if mrGb.reverseIconIndex ~= nil and self.mrGbMS.ReverseActive ~= nil then
							if self.mrGbMS.ReverseActive then
								if not mrGb.reverseIconOn then
									setVisibility(mrGb.reverseIconIndex, true)
									mrGb.reverseIconOn = true
								end
							else
								if mrGb.reverseIconOn then
									setVisibility(mrGb.reverseIconIndex, false)
									mrGb.reverseIconOn = false
								end
							end
						end
						if mrGb.holdIconIndex ~= nil and self.mrGbMS.AutoHold ~= nil then
							if self.mrGbMS.AutoHold then
								if not mrGb.holdIconOn then
									setVisibility(mrGb.holdIconIndex, true)
									mrGb.holdIconOn = true
								end
							else
								if mrGb.holdIconOn then
									setVisibility(mrGb.holdIconIndex, false)
									mrGb.holdIconOn = false
								end
							end
						end
					end
				else
					if self.realCab.gd ~= nil then
						local gd = self.realCab.gd

						if gd.groupIndex ~= nil then
							if not gd.groupIndexOn then
								setVisibility(gd.groupIndex, true)
								gd.groupIndexOn = true
							end
							if gd.neutral ~= nil then
								if dcShuttleOn then
									if gd.neutralOn then
										setVisibility(gd.neutral, false)
										gd.neutralOn = false
									end
								else
									if self:getLastSpeed() * self.speedDisplayScale < 1 and self.movingDirection == 0 then
										if not gd.neutralOn then
											setVisibility(gd.neutral, true)
											gd.neutralOn = true
										end
									else
										if gd.neutralOn then
											setVisibility(gd.neutral, false)
											gd.neutralOn = false
										end
									end
								end
							end
							if gd.reverse ~= nil then
								local showReverse = false
								if dcShuttleOn then
									if self.driveControl.shuttle.direction < 0 then
										showReverse = true
									end
								else
									if self:getLastSpeed() * self.speedDisplayScale > 1 and self.movingDirection < 0 then
										showReverse = true
									end
								end
								if showReverse then
									if not gd.reverseOn then
										setVisibility(gd.reverse, true)
										gd.reverseOn = true
									end
								else
									if gd.reverseOn then
										setVisibility(gd.reverse, false)
										gd.reverseOn = false
									end
								end
							end
							if gd.drive ~= nil then
								local showDrive = false
								if dcShuttleOn then
									if self.driveControl.shuttle.direction > 0 then
										showDrive = true
									end
								else
									if self:getLastSpeed() * self.speedDisplayScale > 1 and self.movingDirection > 0 then
										showDrive = true
									end
								end
								if showDrive then
									if not gd.driveOn then
										setVisibility(gd.drive, true)
										gd.driveOn = true
									end
								else
									if gd.driveOn then
										setVisibility(gd.drive, false)
										gd.driveOn = false
									end
								end
							end
						end
					end
					if self.realCab.mrGb ~= nil then
						local mrGb = self.realCab.mrGb

						if mrGb.groupIndex ~= nil then
							if mrGb.groupIndexOn then
								setVisibility(mrGb.groupIndex, false)
								mrGb.groupIndexOn = false
							end
							if mrGb.reverseIconIndex ~= nil then
								if mrGb.reverseIconOn then
									setVisibility(mrGb.reverseIconIndex, false)
									mrGb.reverseIconOn = false
								end
							end
							if mrGb.holdIconIndex ~= nil then
								if mrGb.holdIconOn then
									setVisibility(mrGb.holdIconIndex, false)
									mrGb.holdIconOn = false
								end
							end
						end
					end
				end
			end
		end
	end
end

function realCab:updateTick(dt)
	if self.realCab.hd ~= nil then
		if g_gameSettings ~= nil and self.realCab.hd.loaded then
			if self.realCab.hd.useMiles ~= g_gameSettings:getValue("useMiles") then
				setVisibility(self.realCab.hd.mphIcon, g_gameSettings:getValue("useMiles"))
				setVisibility(self.realCab.hd.kphIcon, not g_gameSettings:getValue("useMiles"))
				self.realCab.hd.useMiles = g_gameSettings:getValue("useMiles")
			end
		end
	end

	if self.realCab.drl ~= nil then
		if self.isMotorStarted then
			if not self.dayRunLightState then
				self:setDrlState(true)
			end
		else
			if self.dayRunLightState then
				self:setDrlState(false)
			end
		end
	end

	if self.realCab.ca ~= nil then
		if self.realCab.ca.wipersAnimation ~= nil then
			local ca = self.realCab.ca
			if self:getIsActive() and self.isMotorStarted then
				if ca.wipersActivated then
					if not self:getIsAnimationPlaying(ca.wipersAnimation) then
						self:playAnimation(ca.wipersAnimation, ca.wipersSpeed, 0, true)
					end
				end

				if g_currentMission.environment.lastRainScale > 0.1 and g_currentMission.environment.timeSinceLastRain < 30 then
					if not ca.wipersActivated then
						self:playAnimation(ca.wipersAnimation, ca.wipersSpeed, Utils.clamp(self:getAnimationTime(ca.wipersAnimation), 0, 1), true)
						ca.wipersActivated = true
					end
				else
					if ca.wipersActivated then
						self:setAnimationStopTime(ca.wipersAnimation, 1)
						ca.wipersActivated = false
					end
				end
			end
		end
	end

	if self.isClient then
		if self:getIsMotorStarted() then
			if self.realCab.hd ~= nil then
				if self.realCab.hd.bcAnimations ~= nil then
					VehicleHudUtils.setHudValue(self, self.realCab.hd.bcAnimations, self.brakeCompressor.fillLevel, self.brakeCompressor.capacity)
				end
			end
			if self.realCab.mrGb ~= nil and self.mrGbMS ~= nil then
				if self.realCab.mrGb.gearNumber ~= nil and self.realCab.mrGb.groupIndex ~= nil then
					local gearboxIsOn = false
					if self.mrGbMS.IsOn ~= nil then
						gearboxIsOn = self.mrGbMS.IsOn
					end
					if gearboxIsOn and self:mrGbMGetGearNumber() ~= nil then
						VehicleHudUtils.setHudValue(self, self.realCab.mrGb.gearNumber, self:mrGbMGetGearNumber())
					end
				end
			end
		end
	end
end

function realCab:stopMotor()
	if self.realCab.hd ~= nil then
		if self.realCab.hd.bcAnimations ~= nil then
			VehicleHudUtils.setHudValue(self, self.realCab.hd.bcAnimations, 0, self.brakeCompressor.capacity)
		end
	end
	if self.realCab.mrGb ~= nil then
		if self.realCab.mrGb.gearNumber ~= nil then
			VehicleHudUtils.setHudValue(self, self.realCab.mrGb.gearNumber, 0, -1)

			if self.realCab.mrGb.groupIndexOn then
				setVisibility(self.realCab.mrGb.groupIndex, false)
				self.realCab.mrGb.groupIndexOn = false
			end
		end
	end
	if self.realCab.gd ~= nil then
		if self.realCab.gd.groupIndexOn then
			setVisibility(self.realCab.gd.groupIndex, false)
			self.realCab.gd.groupIndexOn = false
		end
	end
end

function realCab:setLightsTypesMask(lightsTypesMask, force, noEventSend)
	if self.realCab.ia ~= nil then
		local ia = self.realCab.ia;
		if ia.lightSwitches ~= nil then
			for _, switch in pairs (ia.lightSwitches) do
				if bitAND(lightsTypesMask, 2^switch.type) ~= 0 then
					if not switch.isActive then
						if switch.useRot then
							setRotation(switch.index, switch.on[1], switch.on[2], switch.on[3] )
						else
							setTranslation(switch.index, switch.on[1], switch.on[2], switch.on[3] )
						end
						switch.isActive = true
					end
				else
					if switch.isActive then
						if switch.useRot then
							setRotation(switch.index, switch.off[1], switch.off[2], switch.off[3] )
						else
							setTranslation(switch.index, switch.off[1], switch.off[2], switch.off[3] )
						end
						switch.isActive = false
					end
				end
			end
		end
	end
end

function realCab:setBeaconLightsVisibility(active, force, noEventSend)
	if self.realCab.ia ~= nil then
		local ia = self.realCab.ia
		if ia.beaconIndex ~= nil then
			if ia.beaconSwitchRot then
				if active then
					setRotation(ia.beaconIndex, ia.beaconOn[1], ia.beaconOn[2], ia.beaconOn[3] )
				else
					setRotation(ia.beaconIndex, ia.beaconOff[1], ia.beaconOff[2], ia.beaconOff[3] )
				end
			else
				if active then
					setTranslation(ia.beaconIndex, ia.beaconOn[1], ia.beaconOn[2], ia.beaconOn[3] )
				else
					setTranslation(ia.beaconIndex, ia.beaconOff[1], ia.beaconOff[2], ia.beaconOff[3] )
				end
			end
		end
	end
end

function realCab:setTurnLightState(state, force, noEventSend)
	if self.realCab.ia ~= nil then
		local ia = self.realCab.ia
		if ia.turnIndex ~= nil then
			if state == Lights.TURNLIGHT_OFF or state == Lights.TURNLIGHT_HAZARD then
				if ia.turnState ~= Lights.TURNLIGHT_OFF then
					setRotation(ia.turnIndex, ia.turnOff[1], ia.turnOff[2], ia.turnOff[3] )
					ia.turnState = Lights.TURNLIGHT_OFF
				end
			else
				if state == Lights.TURNLIGHT_LEFT then
					if ia.turnState ~= Lights.TURNLIGHT_LEFT then
						setRotation(ia.turnIndex, ia.leftTurn[1], ia.leftTurn[2], ia.leftTurn[3] )
						ia.turnState = Lights.TURNLIGHT_LEFT
					end
				elseif state == Lights.TURNLIGHT_RIGHT then
					if ia.turnState ~= Lights.TURNLIGHT_RIGHT then
						setRotation(ia.turnIndex, ia.rightTurn[1], ia.rightTurn[2], ia.rightTurn[3] )
						ia.turnState = Lights.TURNLIGHT_RIGHT
					end
				end
			end
		end

		if ia.hazardIndex ~= nil then
			if state == Lights.TURNLIGHT_HAZARD then
				if ia.beaconState ~= Lights.TURNLIGHT_HAZARD then
					if ia.hazardSwitchRot then
						setRotation(ia.hazardIndex, ia.hazardOn[1], ia.hazardOn[2], ia.hazardOn[3] )
					else
						setTranslation(ia.hazardIndex, ia.hazardOn[1], ia.hazardOn[2], ia.hazardOn[3] )
					end
					ia.beaconState = Lights.TURNLIGHT_HAZARD
				end
			else
				if ia.beaconState == Lights.TURNLIGHT_HAZARD then
					if ia.hazardSwitchRot then
						setRotation(ia.hazardIndex, ia.hazardOff[1], ia.hazardOff[2], ia.hazardOff[3] )
					else
						setTranslation(ia.hazardIndex, ia.hazardOff[1], ia.hazardOff[2], ia.hazardOff[3] )
					end
					ia.beaconState = TURNLIGHT_OFF
				end
			end
		end
	end
end

function realCab:setDrlState(active)
	local highProfile = self:getUseHighProfile()

	for _, drl in ipairs(self.realCab.drl) do
        setVisibility(drl.deco, active)
        if highProfile and drl.hasReal then
			setVisibility(drl.realHigh, active)
        end
    end

	self.dayRunLightState = active
end

function realCab:handbrakeStates(turnOn)
	local ia = self.realCab.ia
	if turnOn then
		if ia.handbrakeRot then
			setRotation(ia.handbrakeIndex, ia.handbrakeOn[1], ia.handbrakeOn[2], ia.handbrakeOn[3] )
		else
			setTranslation(ia.handbrakeIndex, ia.handbrakeOn[1], ia.handbrakeOn[2], ia.handbrakeOn[3] )
		end
	else
		if ia.handbrakeRot then
			setRotation(ia.handbrakeIndex, ia.handbrakeOff[1], ia.handbrakeOff[2], ia.handbrakeOff[3] )
		else
			setTranslation(ia.handbrakeIndex, ia.handbrakeOff[1], ia.handbrakeOff[2], ia.handbrakeOff[3] )
		end
	end

	ia.handbrakeActive = turnOn
end

-- function realCab:vehicleStateCheck()
	-- local wipersActive, gearboxIsOn, dcShuttleOn = false, false, false

	-- if self.realCab.ca.wipersAnimation ~= nil then
		-- wipersActive = self.isMotorStarted and self.realCab.ca.wipersActivated
	-- end

	-- if self.mrGbMS ~= nil and self.mrGbMS.IsOn ~= nil then
		-- gearboxIsOn = self.mrGbMS.IsOn
	-- end

	-- if self.driveControl ~= nil and self.driveControl.shuttle ~= nil then
		-- if self.driveControl.shuttle.isActive then
			-- dcShuttleOn = true
		-- end
	-- end

	-- return wipersActive, gearboxIsOn, dcShuttleOn
-- end

function realCab:draw()
end

function realCab:keyEvent(unicode, sym, modifier, isDown)
end

function realCab:mouseEvent(posX, posY, isDown, isUp, button)
end